/*********************************************************************************************************
* Software License Agreement (BSD License)                                                               *
* Author: Sebastien Decugis <sdecugis@nict.go.jp>							 *
*													 *
* Copyright (c) 2008, WIDE Project and NICT								 *
* All rights reserved.											 *
* 													 *
* Redistribution and use of this software in source and binary forms, with or without modification, are  *
* permitted provided that the following conditions are met:						 *
* 													 *
* * Redistributions of source code must retain the above 						 *
*   copyright notice, this list of conditions and the 							 *
*   following disclaimer.										 *
*    													 *
* * Redistributions in binary form must reproduce the above 						 *
*   copyright notice, this list of conditions and the 							 *
*   following disclaimer in the documentation and/or other						 *
*   materials provided with the distribution.								 *
* 													 *
* * Neither the name of the WIDE Project or NICT nor the 						 *
*   names of its contributors may be used to endorse or 						 *
*   promote products derived from this software without 						 *
*   specific prior written permission of WIDE Project and 						 *
*   NICT.												 *
* 													 *
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED *
* WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A *
* PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR *
* ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 	 *
* LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 	 *
* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR *
* TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF   *
* ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.								 *
*********************************************************************************************************/

/* Header file for the rt_default extension. 
 *
 *  This extension provides the default base Diameter routing mechanisms:
 * - Send message to the peer listed as Destination-Host if available.
 * - Send message to peers belonging to the Destinatino-Realm is available.
 *   If several such peers are available, configuration file can be used to specify preferences.
 * - Send message to configured default peers in other cases.
 *
 *  See the rt_default.conf.sample file for the format of the configuration file.
 */
 
#define IN_EXTENSION
#define DEFINE_DEBUG_MACRO  rt_default
#include <waaad/waaad.h>

#include <pthread.h>
#include <stdlib.h>
#include <string.h>

/* debug level */
extern int rt_default_verbosity;	/* default: 0 */
	
/* The module configuration */
typedef struct {
	/* Destination-Host management? */
	int	do_dest_host;	/* default: 1 */
	int	do_dest_realm;	/* default: 1 */
	
	/* The scores given to peers depending on their matchs: */
	int	sc_dest_host;	/* default: PEER_SCORE_FINALDEST */
	int	sc_dest_realm;	/* default: PEER_SCORE_DEFAULT_REALM */
	
	int	sc_no_info;	/* default: 0 */
	int	sc_all;		/* default: PEER_SCORE_DEFAULT */
	int	sc_all_pref;	/* default: PEER_SCORE_DEFAULT + PEER_SCORE_LOAD_BALANCE */
	int	sc_rlm;		/* default: PEER_SCORE_DEFAULT_REALM */
	int	sc_rlm_pref;	/* default: PEER_SCORE_DEFAULT_REALM + PEER_SCORE_LOAD_BALANCE */
} rtd_conf_t;
extern rtd_conf_t * rtd_conf;


/* Parse the configuration file */
int rtd_conf_handle(char * conffile);


/* Initialize the peer table */
int rtd_init(void);

/* Destroy the peer table */
void rtd_fini(void);

/* Add an entry (from configuration file) */
int rtd_add(char * realm /* or NULL for default */, char * peer /* the diameter id */, int prefered /* 0 or 1 */);

/* Search the score for a given realm & peer */
int rtd_search( unsigned char * realm /* destination realm of the msg */, size_t realm_len, 
		char * peer, int * score);

/* For debug: dump the table */
void rtd_dump(void);
